@@ -3,14 +3,15 @@ use super::{Args, Result};
 use crate::interpreter::RuntimeErrorKind;
 use crate::quantity::Quantity;
 use crate::value::Value;
+use crate::vm::ExecutionContext;
 
-pub fn len(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn len(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let list = list_arg!(args);
 
     return_scalar!(list.len() as f64)
 }
 
-pub fn head(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn head(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let list = list_arg!(args);
 
     if let Some(first) = list.head() {
@@ -20,14 +21,14 @@ pub fn head(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     }
 }
 
-pub fn tail(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn tail(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let mut list = list_arg!(args);
 
     list.tail()?;
     Ok(list.into())
 }
 
-pub fn cons(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn cons(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     let element = arg!(args);
     let mut list = list_arg!(args);
     list.push_front(element);
@@ -35,7 +36,10 @@ pub fn cons(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
     return_list!(list)
 }
 
-pub fn cons_end(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn cons_end(
+    _ctx: &mut ExecutionContext,
+    mut args: Args,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let element = arg!(args);
     let mut list = list_arg!(args);
     list.push_back(element);
