@@ -7,8 +7,12 @@ use crate::interpreter::RuntimeErrorKind;
 use crate::quantity::Quantity;
 use crate::typed_ast::DType;
 use crate::value::Value;
+use crate::vm::ExecutionContext;
 
-pub fn _get_chemical_element_data_raw(mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn _get_chemical_element_data_raw(
+    _ctx: &mut ExecutionContext,
+    mut args: Args,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     use crate::span::{ByteIndex, Span};
     use crate::typed_ast::Type;
     use crate::typed_ast::{StructInfo, StructKind};
