@@ -13,6 +13,7 @@ use std::collections::VecDeque;
 
 use crate::interpreter::{RuntimeError, RuntimeErrorKind};
 use crate::span::Span;
+use crate::typechecker::type_scheme::TypeScheme;
 use crate::value::Value;
 use crate::vm::ExecutionContext;
 
@@ -22,11 +23,17 @@ pub(crate) type ArityRange = std::ops::RangeInclusive<usize>;
 
 type Result<T, E = Box<RuntimeError>> = std::result::Result<T, E>;
 
-pub(crate) type Args = VecDeque<Value>;
+pub(crate) struct Arg {
+    pub value: Value,
+    pub type_: TypeScheme,
+    pub span: Span,
+}
+
+pub(crate) type Args = VecDeque<Arg>;
 
 pub(crate) enum Callable {
-    Function(fn(Args) -> Result<Value, Box<RuntimeErrorKind>>),
-    Procedure(fn(&mut ExecutionContext, Args, Vec<Span>) -> ControlFlow),
+    Function(fn(&mut ExecutionContext, Args) -> Result<Value, Box<RuntimeErrorKind>>),
+    Procedure(fn(&mut ExecutionContext, Args) -> ControlFlow),
 }
 
 pub(crate) struct ForeignFunction {
