@@ -1,5 +1,5 @@
+use crate::dimension::DimensionRegistryError;
 use crate::ffi::ArityRange;
-use crate::registry::RegistryError;
 use crate::span::Span;
 use crate::typed_ast::BinaryOperator;
 use crate::{BaseRepresentation, NameResolutionError, Type};
@@ -31,7 +31,7 @@ pub enum TypeCheckError {
     DivisionByZeroInConstEvalExpression(Span),
 
     #[error("{0}")]
-    RegistryError(RegistryError),
+    DimensionRegistryError(#[from] DimensionRegistryError),
 
     #[error("Incompatible alternative expressions have been provided for dimension '{0}'")]
     IncompatibleAlternativeDimensionExpression(
@@ -115,6 +115,14 @@ pub enum TypeCheckError {
     #[error("Unknown struct '{1}")]
     UnknownStruct(Span, String),
 
+    #[error("Wrong number of type arguments for '{type_name}': expected {expected}, got {actual}")]
+    WrongNumberOfTypeArguments {
+        span: Span,
+        type_name: String,
+        expected: usize,
+        actual: usize,
+    },
+
     #[error("Field '{2}' does not exist in struct '{3}'")]
     UnknownFieldInStructInstantiation(Span, Span, String, String),
 
