@@ -6,12 +6,14 @@ use compact_str::{CompactString, ToCompactString};
 use indexmap::IndexMap;
 use num_traits::ToPrimitive;
 
+use crate::dimension::DimensionRegistry;
 use crate::interpreter::RuntimeErrorKind;
 use crate::list::NumbatList;
 use crate::span::Span;
+use crate::typechecker::type_scheme::TypeScheme;
 use crate::typed_ast::StructInfo;
 use crate::{
-    ffi::{self, ArityRange, Callable, ForeignFunction},
+    ffi::{self, Arg, ArityRange, Callable, ForeignFunction},
     interpreter::{InterpreterResult, PrintFunction, Result, RuntimeError},
     markup::Markup,
     math,
@@ -269,6 +271,14 @@ impl CallFrame {
 
 pub struct ExecutionContext<'a> {
     pub print_fn: &'a mut PrintFunction,
+    pub dimension_registry: &'a DimensionRegistry,
+}
+
+/// Metadata for a single FFI call argument
+#[derive(Clone)]
+pub struct FfiCallArg {
+    pub span: Span,
+    pub type_: TypeScheme,
 }
 
 #[derive(Clone)]
@@ -304,9 +314,8 @@ pub struct Vm {
     /// List of registered native/foreign functions
     ffi_callables: IndexMap<&'static str, &'static ForeignFunction>,
 
-    /// Spans for arguments of procedure calls. This is used for
-    /// assertion error messages, for example.
-    procedure_arg_spans: Vec<Vec<Span>>,
+    /// Metadata for FFI call arguments (spans and types).
+    ffi_call_args: Vec<FfiCallArg>,
 
     /// The call stack
     frames: Vec<CallFrame>,
@@ -335,7 +344,7 @@ impl Vm {
                 .iter()
                 .map(|(kind, ff)| (kind.name(), ff))
                 .collect(),
-            procedure_arg_spans: vec![],
+            ffi_call_args: vec![],
             frames: vec![CallFrame::root()],
             stack: vec![],
             debug: false,
@@ -502,10 +511,11 @@ impl Vm {
         Some(position as u16)
     }
 
-    pub(crate) fn add_procedure_arg_span(&mut self, spans: Vec<Span>) -> u16 {
-        self.procedure_arg_spans.push(spans);
-        assert!(self.procedure_arg_spans.len() <= u16::MAX as usize);
-        (self.procedure_arg_spans.len() - 1) as u16
+    pub(crate) fn add_ffi_call_args(&mut self, args: Vec<FfiCallArg>) -> u16 {
+        let start_idx = self.ffi_call_args.len();
+        self.ffi_call_args.extend(args);
+        assert!(self.ffi_call_args.len() <= u16::MAX as usize);
+        start_idx as u16
     }
 
     pub fn disassemble(&self) {
@@ -885,25 +895,31 @@ impl Vm {
                 Op::FFICallFunction | Op::FFICallProcedure => {
                     let function_idx = self.read_u16() as usize;
                     let num_args = self.read_u16() as usize;
+                    let call_args_start = self.read_u16() as usize;
                     let foreign_function = &self.ffi_callables[function_idx];
 
                     debug_assert!(foreign_function.arity.contains(&num_args));
 
+                    let call_args =
+                        self.ffi_call_args[call_args_start..call_args_start + num_args].to_vec();
                     let mut args = VecDeque::new();
-                    for _ in 0..num_args {
-                        args.push_front(self.pop());
+                    for i in 0..num_args {
+                        let call_arg = &call_args[num_args - 1 - i];
+                        args.push_front(Arg {
+                            value: self.pop(),
+                            type_: call_arg.type_.clone(),
+                            span: call_arg.span,
+                        });
                     }
 
                     match &self.ffi_callables[function_idx].callable {
                         Callable::Function(function) => {
-                            let result = (function)(args).map_err(|e| self.runtime_error(*e))?;
+                            let result =
+                                (function)(ctx, args).map_err(|e| self.runtime_error(*e))?;
                             self.push(result);
                         }
                         Callable::Procedure(procedure) => {
-                            let span_idx = self.read_u16() as usize;
-                            let spans = &self.procedure_arg_spans[span_idx];
-
-                            let result = (procedure)(ctx, args, spans.clone());
+                            let result = (procedure)(ctx, args);
 
                             match result {
                                 std::ops::ControlFlow::Continue(()) => {}
@@ -916,6 +932,7 @@ impl Vm {
                 }
                 Op::CallCallable => {
                     let num_args = self.read_u16() as usize;
+                    let call_args_start = self.read_u16() as usize;
 
                     let callable = self.pop();
                     match callable.unsafe_as_function_reference() {
@@ -935,15 +952,23 @@ impl Vm {
                                 .expect("Foreign function exists")
                                 as usize;
 
+                            let call_args = self.ffi_call_args
+                                [call_args_start..call_args_start + num_args]
+                                .to_vec();
                             let mut args = VecDeque::new();
-                            for _ in 0..num_args {
-                                args.push_front(self.pop());
+                            for i in 0..num_args {
+                                let call_arg = &call_args[num_args - 1 - i];
+                                args.push_front(Arg {
+                                    value: self.pop(),
+                                    type_: call_arg.type_.clone(),
+                                    span: call_arg.span,
+                                });
                             }
 
                             match &self.ffi_callables[function_idx].callable {
                                 Callable::Function(function) => {
-                                    let result =
-                                        (function)(args).map_err(|e| self.runtime_error(*e))?;
+                                    let result = (function)(ctx, args)
+                                        .map_err(|e| self.runtime_error(*e))?;
                                     self.push(result);
                                 }
                                 Callable::Procedure(..) => unreachable!(
@@ -1151,8 +1176,10 @@ fn vm_basic() {
     vm.add_op(Op::Return, Span::dummy());
 
     let mut print_fn = |_: &Markup| {};
+    let dimension_registry = DimensionRegistry::default();
     let mut ctx = ExecutionContext {
         print_fn: &mut print_fn,
+        dimension_registry: &dimension_registry,
     };
 
     assert_eq!(
