@@ -881,7 +881,7 @@ fn test_full_simplify_for_function_calls() {
 fn test_datetime_runtime_errors() {
     expect_failure(
         "datetime(\"2000-01-99\")",
-        "Unrecognized datetime format: failed to parse day in date \"2000-01-99\": day is not valid: parameter 'day' with value 99 is not in the required range of 1..=31",
+        "Unrecognized datetime format: failed to parse day in date: parsed day is not valid: parameter 'day' with value 99 is not in the required range of 1..=31",
     );
     expect_failure("now() -> tz(\"Europe/NonExisting\")", "Unknown timezone");
     expect_failure(
@@ -894,7 +894,12 @@ fn test_datetime_runtime_errors() {
     );
     expect_failure(
         "format_datetime(\"%Y %;\", now())",
-        "strftime formatting failed: found unrecognized specifier directive %;.",
+        "strftime formatting failed: found unrecognized specifier directive `;`.",
+    );
+    // Regression test for https://github.com/sharkdp/numbat/issues/801
+    expect_failure(
+        "format_datetime(\"%:\", now())",
+        "strftime formatting failed: expected to find specifier directive after colons",
     );
 }
 
